from mozilla_django_oidc.auth import OIDCAuthenticationBackend

from .signals import post_login


class PiratesOIDCAuthenticationBackend(OIDCAuthenticationBackend):
    """
    Pirates OIDC Authentication Backend.

    Instead of `email` uses claim `sub` (as `sso_id`) to identify users. Which
    allows for email change.
    """

    def get_sso_id(self, claims):
        return claims.get("sub")

    def filter_users_by_claims(self, claims):
        sso_id = self.get_sso_id(claims)
        if not sso_id:
            return self.UserModel.objects.none()
        return self.UserModel.objects.filter(sso_id=sso_id)

    def create_user(self, claims):
        sso_id = self.get_sso_id(claims)
        first_name = claims.get("given_name", "")
        last_name = claims.get("family_name", "")
        email = claims.get("email", "")
        user = self.UserModel.objects.create(
            sso_id=sso_id, first_name=first_name, last_name=last_name, email=email
        )
        self.send_post_login_signal(user, True, claims)
        return user

    def update_user(self, user, claims):
        user.first_name = claims.get("given_name", "")
        user.last_name = claims.get("family_name", "")
        user.email = claims.get("email", "")
        user.save()
        self.send_post_login_signal(user, False, claims)
        return user

    def send_post_login_signal(self, user, created, claims):
        post_login.send(
            sender=self.__class__,
            user=user,
            created=created,
            claims=claims,
            request=self.request,
        )



from django.conf import settings
from django.contrib.auth.models import PermissionsMixin
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _


class AbstractUser(PermissionsMixin):
    """
    An abstract base class implementing a fully featured User model with
    admin-compliant permissions.
    """

    sso_id = models.CharField(
        _("SSO ID"),
        max_length=150,
        unique=True,
        error_messages={"unique": _("A user with that SSO ID already exists.")},
    )
    first_name = models.CharField(_("first name"), max_length=150, blank=True)
    last_name = models.CharField(_("last name"), max_length=150, blank=True)
    email = models.EmailField(_("email address"), blank=True)
    is_staff = models.BooleanField(
        _("staff status"),
        default=False,
        help_text=_("Designates whether the user can log into this admin site."),
    )
    is_active = models.BooleanField(
        _("active"),
        default=True,
        help_text=_(
            "Designates whether this user should be treated as active. "
            "Unselect this instead of deleting accounts."
        ),
    )
    date_joined = models.DateTimeField(_("date joined"), default=timezone.now)

    USERNAME_FIELD = "sso_id"
    REQUIRED_FIELDS = []

    class Meta:
        abstract = True
        verbose_name = _("user")
        verbose_name_plural = _("users")

    def get_full_name(self):
        """Return the first_name plus the last_name, with a space in between."""
        full_name = f"{self.first_name} {self.last_name}"
        return full_name.strip()

    def get_short_name(self):
        """Return the short name for the user."""
        return self.first_name

    def __str__(self):
        return self.get_full_name() or self.email or self.sso_id

    @property
    def is_anonymous(self):
        """
        Always return False. This is a way of comparing User objects to
        anonymous users.
        """
        return False

    @property
    def is_authenticated(self):
        """
        Always return True. This is a way to tell if the user has been
        authenticated in templates.
        """
        return True


class AbstractTeam(models.Model):
    name = models.CharField(max_length=250, unique=True)
    members = models.ManyToManyField(settings.AUTH_USER_MODEL)

    class Meta:
        abstract = True
        ordering = ["name"]
        verbose_name = _("team")
        verbose_name_plural = _("teams")

    def __str__(self):
        return self.name


class AbstractOrgGroup(models.Model):
    name = models.CharField(max_length=250, unique=True)
    members = models.ManyToManyField(settings.AUTH_USER_MODEL)

    class Meta:
        abstract = True
        ordering = ["name"]
        verbose_name = _("group")
        verbose_name_plural = _("groups")

    def __str__(self):
        return self.name



#!/usr/bin/env python

import codecs
import os

from setuptools import setup


def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding="utf-8").read()


setup(
    name="pirates",
    version="0.5.0",
    license="MIT",
    description="Django app for users, teamds and groups.",
    long_description=read("README.md"),
    long_description_content_type="text/markdown",
    author="Jan Bednařík",
    author_email="jan.bednarik@gmail.com",
    url="https://gitlab.pirati.cz/to/pirates",
    packages=["pirates"],
    include_package_data=True,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Topic :: Utilities",
    ],
    project_urls={
        # "Documentation": "https://pirates.readthedocs.io/",
        # "Changelog": "https://pirates.readthedocs.io/en/latest/changelog.html",
        "Issue Tracker": "https://gitlab.pirati.cz/to/pirates/issues",
    },
    keywords=["django", "openid", "sso"],
    python_requires=">=3.6",
    install_requires=["mozilla-django-oidc>=1.2.4,<2"],
)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class PiratesConfig(AppConfig):
    name = "pirates"
    verbose_name = _("Pirates")

    def ready(self):
        try:
            import pirates.signals
        except ImportError:
            pass
